@@ -34,6 +34,7 @@ pub struct Context {
     pub use_pty: bool,
     pub noexec: bool,
     pub umask: Umask,
+    pub noninteractive_auth: bool,
     // sudoedit
     #[cfg_attr(not(feature = "sudoedit"), allow(unused))]
     pub files_to_edit: Vec<Option<SudoPath>>,
@@ -86,6 +87,8 @@ impl Context {
             })?
         };
 
+        let prompt = sudo_options.prompt.or_else(|| env::var("SUDO_PROMPT").ok());
+
         Ok(Context {
             hostname,
             command,
@@ -97,12 +100,13 @@ impl Context {
             chdir: sudo_options.chdir,
             stdin: sudo_options.stdin,
             bell: sudo_options.bell,
-            prompt: sudo_options.prompt,
+            prompt,
             non_interactive: sudo_options.non_interactive,
             process: Process::new(),
             use_pty: true,
             noexec: false,
             umask: Umask::Preserve,
+            noninteractive_auth: false,
             files_to_edit: vec![],
         })
     }
@@ -174,6 +178,7 @@ impl Context {
             use_pty: true,
             noexec: false,
             umask: Umask::Preserve,
+            noninteractive_auth: false,
             files_to_edit,
         })
     }
@@ -200,6 +205,7 @@ impl Context {
             use_pty: true,
             noexec: false,
             umask: Umask::Preserve,
+            noninteractive_auth: false,
             files_to_edit: vec![],
         })
     }
@@ -249,6 +255,7 @@ impl Context {
             use_pty: true,
             noexec: false,
             umask: Umask::Preserve,
+            noninteractive_auth: false,
             files_to_edit: vec![],
         })
     }
