@@ -10,7 +10,7 @@ use crate::exec::{cond_fmt, handle_sigchld, signal_fmt, terminate_process, Handl
 use crate::exec::{
     io_util::retry_while_interrupted,
     use_pty::backchannel::{BackchannelPair, MonitorMessage, ParentBackchannel, ParentMessage},
-    ExitReason,
+    ExitReason, SpawnNoexecHandler,
 };
 use crate::log::{dev_error, dev_info, dev_warn};
 use crate::system::signal::{
@@ -19,9 +19,7 @@ use crate::system::signal::{
 };
 use crate::system::term::{Pty, PtyFollower, PtyLeader, TermSize, Terminal, UserTerm};
 use crate::system::wait::WaitOptions;
-use crate::system::{
-    chown, fork, getpgrp, kill, killpg, FileCloser, ForkResult, Group, User, _exit,
-};
+use crate::system::{chown, fork, getpgrp, kill, killpg, ForkResult, Group, User, _exit};
 use crate::system::{getpgid, interface::ProcessId};
 
 use super::pipe::Pipe;
@@ -29,6 +27,7 @@ use super::{CommandStatus, SIGCONT_BG};
 
 pub(in crate::exec) fn exec_pty(
     sudo_pid: ProcessId,
+    spawn_noexec_handler: Option<SpawnNoexecHandler>,
     mut command: Command,
     user_tty: UserTerm,
 ) -> io::Result<ExitReason> {
@@ -58,18 +57,12 @@ pub(in crate::exec) fn exec_pty(
     // Fetch the parent process group so we can signals to it.
     let parent_pgrp = getpgrp();
 
-    let mut file_closer = FileCloser::new();
-
     // Set all the IO streams for the command to the follower side of the pty.
-    let mut clone_follower = || -> io::Result<PtyFollower> {
-        let follower = pty.follower.try_clone().map_err(|err| {
+    let clone_follower = || -> io::Result<PtyFollower> {
+        pty.follower.try_clone().map_err(|err| {
             dev_error!("cannot clone pty follower: {err}");
             err
-        })?;
-        // Don't close these as we will need them so they are dupped inside `Command::exec`.
-        file_closer.except(&follower);
-
-        Ok(follower)
+        })
     };
 
     command.stdin(clone_follower()?);
@@ -94,7 +87,7 @@ pub(in crate::exec) fn exec_pty(
         .tcgetpgrp()
         .is_ok_and(|tty_pgrp| tty_pgrp == parent_pgrp);
     dev_info!(
-        "sudo is runnning in the {}",
+        "sudo is running in the {}",
         cond_fmt(foreground, "foreground", "background")
     );
 
@@ -135,6 +128,12 @@ pub(in crate::exec) fn exec_pty(
         command.stderr(Stdio::inherit());
     }
 
+    // If there is another process later in the pipeline, don't interfere
+    // with its access to the Tty
+    if io::stdout().is_pipe() {
+        foreground = false;
+    }
+
     // Copy terminal settings from `/dev/tty` to the pty.
     if let Err(err) = user_tty.copy_to(&pty.follower) {
         dev_error!("cannot copy terminal settings to pty: {err}");
@@ -165,6 +164,10 @@ pub(in crate::exec) fn exec_pty(
         }
     };
 
+    if !foreground {
+        tty_pipe.disable_input(&mut registry);
+    }
+
     // SAFETY: There should be no other threads at this point.
     let ForkResult::Parent(monitor_pid) = (unsafe { fork() }).map_err(|err| {
         dev_error!("cannot fork monitor process: {err}");
@@ -181,7 +184,6 @@ pub(in crate::exec) fn exec_pty(
             command,
             foreground && !pipeline && !exec_bg,
             &mut backchannels.monitor,
-            file_closer,
             original_set,
         ) {
             Ok(exec_output) => match exec_output {},
@@ -206,17 +208,19 @@ pub(in crate::exec) fn exec_pty(
         _exit(1);
     };
 
+    if let Some(spawner) = spawn_noexec_handler {
+        spawner.spawn();
+    }
+
     // Close the file descriptors that we don't access
     drop(pty.follower);
     drop(backchannels.monitor);
 
     // Send green light to the monitor after closing the follower.
-    retry_while_interrupted(|| backchannels.parent.send(&MonitorMessage::ExecCommand)).map_err(
-        |err| {
-            dev_error!("cannot send green light to monitor: {err}");
-            err
-        },
-    )?;
+    retry_while_interrupted(|| backchannels.parent.send(&MonitorMessage::Edge)).map_err(|err| {
+        dev_error!("cannot send green light to monitor: {err}");
+        err
+    })?;
 
     let mut closure = ParentClosure::new(
         monitor_pid,
@@ -241,6 +245,7 @@ pub(in crate::exec) fn exec_pty(
     // FIXME (ogsudo): Retry if `/dev/tty` is revoked.
 
     // Flush the terminal
+    closure.tty_pipe.right().set_nonblocking()?;
     closure.tty_pipe.flush_left().ok();
 
     // Restore the terminal settings
@@ -353,10 +358,19 @@ impl ParentClosure {
     }
 
     fn run(&mut self, registry: EventRegistry<Self>) -> io::Result<ExitReason> {
-        match registry.event_loop(self) {
+        let result = match registry.event_loop(self) {
             StopReason::Break(err) | StopReason::Exit(ParentExit::Backchannel(err)) => Err(err),
             StopReason::Exit(ParentExit::Command(exit_reason)) => Ok(exit_reason),
-        }
+        };
+        // Send red light to the monitor after processing all events
+        retry_while_interrupted(|| self.backchannel.send(&MonitorMessage::Edge)).map_err(
+            |err| {
+                dev_error!("cannot send red light to monitor: {err}");
+                err
+            },
+        )?;
+
+        result
     }
 
     /// Read an event from the backchannel and return the event if it should break the event loop.
@@ -577,7 +591,7 @@ impl ParentClosure {
         Ok(())
     }
 
-    /// Restore the terminal when sudo resumes after receving `SIGCONT`.
+    /// Restore the terminal when sudo resumes after receiving `SIGCONT`.
     fn resume_terminal(&mut self) -> io::Result<()> {
         self.check_foreground()?;
 
