@@ -1,22 +1,22 @@
-#![forbid(unsafe_code)]
+#![deny(unsafe_code)]
 
 use crate::common::resolve::CurrentUser;
 use crate::common::Error;
 use crate::log::dev_info;
 use crate::system::interface::UserId;
-use crate::system::kernel::kernel_check;
 use crate::system::timestamp::RecordScope;
 use crate::system::User;
 use crate::system::{time::Duration, timestamp::SessionRecordFile, Process};
-use cli::help;
 #[cfg(test)]
 pub(crate) use cli::SudoAction;
 #[cfg(not(test))]
 use cli::SudoAction;
-use std::path::Path;
+use std::path::PathBuf;
 
 mod cli;
-pub(crate) use cli::{SudoListOptions, SudoRunOptions, SudoValidateOptions};
+pub(crate) use cli::{SudoEditOptions, SudoListOptions, SudoRunOptions, SudoValidateOptions};
+#[cfg(feature = "sudoedit")]
+mod edit;
 
 pub(crate) mod diagnostic;
 mod env;
@@ -46,18 +46,19 @@ const VERSION: &str = if let Some(version_override) = std::option_env!("SUDO_RS_
     std::env!("CARGO_PKG_VERSION")
 };
 
-pub(crate) fn candidate_sudoers_file() -> &'static Path {
-    let pb_rs = Path::new("/etc/sudoers-rs");
-    let file = if pb_rs.exists() {
-        pb_rs
-    } else if cfg!(target_os = "freebsd") {
-        // FIXME maybe make this configurable by the packager?
-        Path::new("/usr/local/etc/sudoers")
+pub(crate) fn candidate_sudoers_file() -> PathBuf {
+    let mut path = if cfg!(target_os = "freebsd") {
+        option_env!("LOCALBASE").unwrap_or("/usr/local").into()
     } else {
-        Path::new("/etc/sudoers")
+        PathBuf::from("/")
     };
-    dev_info!("Running with {} file", file.display());
-    file
+    path.push("etc/sudoers-rs");
+    if !path.exists() {
+        path.set_file_name("sudoers");
+    };
+
+    dev_info!("Running with {} file", path.display());
+    path
 }
 
 fn sudo_process() -> Result<(), Error> {
@@ -66,13 +67,22 @@ fn sudo_process() -> Result<(), Error> {
     dev_info!("development logs are enabled");
 
     self_check()?;
-    kernel_check()?;
+
+    let usage_msg: &str;
+    let long_help: fn() -> String;
+    if cli::is_sudoedit(std::env::args().next()) {
+        usage_msg = cli::help_edit::USAGE_MSG;
+        long_help = cli::help_edit::long_help_message;
+    } else {
+        usage_msg = cli::help::USAGE_MSG;
+        long_help = cli::help::long_help_message;
+    }
 
     // parse cli options
     match SudoAction::from_env() {
         Ok(action) => match action {
             SudoAction::Help(_) => {
-                eprintln_ignore_io_error!("{}", help::long_help_message());
+                eprintln_ignore_io_error!("{}", long_help());
                 std::process::exit(0);
             }
             SudoAction::Version(_) => {
@@ -91,7 +101,7 @@ fn sudo_process() -> Result<(), Error> {
                     let user = CurrentUser::resolve()?;
                     let mut record_file =
                         SessionRecordFile::open_for_user(&user, Duration::seconds(0))?;
-                    record_file.disable(scope, None)?;
+                    record_file.disable(scope)?;
                 }
                 Ok(())
             }
@@ -99,7 +109,7 @@ fn sudo_process() -> Result<(), Error> {
             SudoAction::Run(options) => {
                 // special case for when no command is given
                 if options.positional_args.is_empty() && !options.shell && !options.login {
-                    eprintln_ignore_io_error!("{}", help::USAGE_MSG);
+                    eprintln_ignore_io_error!("{}", usage_msg);
                     std::process::exit(1);
                 } else {
                     #[cfg(feature = "dev")]
@@ -109,13 +119,16 @@ fn sudo_process() -> Result<(), Error> {
                 }
             }
             SudoAction::List(options) => pipeline::run_list(options),
+            #[cfg(feature = "sudoedit")]
+            SudoAction::Edit(options) => pipeline::run_edit(options),
+            #[cfg(not(feature = "sudoedit"))]
             SudoAction::Edit(_) => {
                 eprintln_ignore_io_error!("error: `--edit` flag has not yet been implemented");
                 std::process::exit(1);
             }
         },
         Err(e) => {
-            eprintln_ignore_io_error!("{e}\n{}", help::USAGE_MSG);
+            eprintln_ignore_io_error!("{e}\n{}", usage_msg);
             std::process::exit(1);
         }
     }
