@@ -287,7 +287,7 @@ pub(super) fn parse_list<T: Parse + UserFriendly>(
 }
 
 /// Types that implement the Many trait can be parsed multiple tokens into a `Vec<T>`; they are
-/// seperated by `SEP`. There should also be a limit on the number of items.
+/// separated by `SEP`. There should also be a limit on the number of items.
 pub trait Many {
     const SEP: char = ',';
     const LIMIT: usize = 127;
@@ -348,7 +348,7 @@ fn expect_complete<T: Parse>(stream: &mut CharStream) -> Parsed<T> {
 /// AST constructors by hand.
 #[cfg(test)]
 pub fn parse_string<T: Parse>(text: &str) -> Parsed<T> {
-    expect_complete(&mut CharStream::new(text.chars()))
+    expect_complete(&mut CharStream::new(text))
 }
 
 #[cfg(test)]
@@ -384,7 +384,7 @@ mod test {
 
     #[test]
     fn lines_test() {
-        let input = |text: &str| parse_lines(&mut CharStream::new(text.chars()));
+        let input = |text: &str| parse_lines(&mut CharStream::new(text));
 
         let s = |text: &str| Ok(text.to_string());
         assert_eq!(input("hello\nworld\n"), vec![s("hello"), s("world")]);
