@@ -18,7 +18,6 @@ use crate::{
     sudo::{candidate_sudoers_file, diagnostic},
     sudoers::{self, Sudoers},
     system::{
-        can_execute,
         file::{create_temporary_dir, Chown, FileLock},
         interface::{GroupId, UserId},
         signal::{consts::*, register_handlers, SignalStream},
@@ -79,9 +78,9 @@ pub fn main() {
 }
 
 fn check(file_arg: Option<&str>, perms: bool, owner: bool) -> io::Result<()> {
-    let sudoers_path = file_arg
-        .map(Path::new)
-        .unwrap_or_else(|| candidate_sudoers_file());
+    let sudoers_path = &file_arg
+        .map(PathBuf::from)
+        .unwrap_or_else(candidate_sudoers_file);
 
     let sudoers_file = File::open(sudoers_path)
         .map_err(|err| io_msg!(err, "unable to open {}", sudoers_path.display()))?;
@@ -138,21 +137,43 @@ fn check(file_arg: Option<&str>, perms: bool, owner: bool) -> io::Result<()> {
 }
 
 fn run(file_arg: Option<&str>, perms: bool, owner: bool) -> io::Result<()> {
-    let sudoers_path = file_arg
-        .map(Path::new)
-        .unwrap_or_else(|| candidate_sudoers_file());
+    let sudoers_path = &file_arg
+        .map(PathBuf::from)
+        .unwrap_or_else(candidate_sudoers_file);
 
     let (sudoers_file, existed) = if sudoers_path.exists() {
-        let file = File::options().read(true).write(true).open(sudoers_path)?;
+        let file = File::options()
+            .read(true)
+            .write(true)
+            .open(sudoers_path)
+            .map_err(|e| {
+                io::Error::new(
+                    e.kind(),
+                    format!("Failed to open existing sudoers file at {sudoers_path:?}: {e}"),
+                )
+            })?;
 
         (file, true)
     } else {
         // Create a sudoers file if it doesn't exist.
-        let file = File::create(sudoers_path)?;
+        let file = File::create(sudoers_path).map_err(|e| {
+            io::Error::new(
+                e.kind(),
+                format!("Failed to create sudoers file at {sudoers_path:?}: {e}"),
+            )
+        })?;
         // ogvisudo sets the permissions of the file so it can be read and written by the user and
         // read by the group if the `-f` argument was passed.
         if file_arg.is_some() {
-            file.set_permissions(Permissions::from_mode(0o640))?;
+            file.set_permissions(Permissions::from_mode(0o640))
+                .map_err(|e| {
+                    io::Error::new(
+                        e.kind(),
+                        format!(
+                            "Failed to set permissions on new sudoers file at {sudoers_path:?}: {e}"
+                        ),
+                    )
+                })?;
         }
         (file, false)
     };
@@ -200,7 +221,7 @@ fn run(file_arg: Option<&str>, perms: bool, owner: bool) -> io::Result<()> {
         .truncate(true)
         .open(&tmp_path)?;
 
-    tmp_file.set_permissions(Permissions::from_mode(0o700))?;
+    tmp_file.set_permissions(Permissions::from_mode(0o600))?;
 
     let result = edit_sudoers_file(
         existed,
@@ -226,7 +247,6 @@ fn edit_sudoers_file(
 ) -> io::Result<()> {
     let mut stderr = io::stderr();
 
-    let mut editor_path = None;
     let mut sudoers_contents = Vec::new();
 
     // Since visudo is meant to run as root, resolve shouldn't fail
@@ -240,7 +260,7 @@ fn edit_sudoers_file(
 
     let host_name = Hostname::resolve();
 
-    if existed {
+    let editor_path = if existed {
         // If the sudoers file existed, read its contents and write them into the temporary file.
         sudoers_file.read_to_end(&mut sudoers_contents)?;
         // Rewind the sudoers file so it can be written later.
@@ -248,23 +268,28 @@ fn edit_sudoers_file(
         // Write to the temporary file.
         tmp_file.write_all(&sudoers_contents)?;
 
-        let (sudoers, errors) = Sudoers::read(sudoers_contents.as_slice(), sudoers_path)?;
+        let (sudoers, _errors) = Sudoers::read(sudoers_contents.as_slice(), sudoers_path)?;
 
-        if errors.is_empty() {
-            editor_path = sudoers.solve_editor_path(&host_name, &current_user, &current_user);
-        }
-    }
-
-    let editor_path = match editor_path {
-        Some(path) => path,
-        None => editor_path_fallback()?,
+        sudoers.visudo_editor_path(&host_name, &current_user, &current_user)
+    } else {
+        // there is no /etc/sudoers config yet, so use a system default
+        PathBuf::from(crate::defaults::SYSTEM_EDITOR)
     };
 
     loop {
         Command::new(&editor_path)
             .arg("--")
             .arg(tmp_path)
-            .spawn()?
+            .spawn()
+            .map_err(|_| {
+                io::Error::new(
+                    io::ErrorKind::NotFound,
+                    format!(
+                        "specified editor ({}) could not be used",
+                        editor_path.display()
+                    ),
+                )
+            })?
             .wait_with_output()?;
 
         let (sudoers, errors) = File::open(tmp_path)
@@ -332,27 +357,6 @@ fn edit_sudoers_file(
     Ok(())
 }
 
-fn editor_path_fallback() -> io::Result<PathBuf> {
-    let editors = if cfg!(target_os = "linux") {
-        &["/usr/bin/editor"][..]
-    } else if cfg!(target_os = "freebsd") {
-        &["/usr/bin/vi"]
-    } else {
-        unimplemented!("unknown default editor for target")
-    };
-    for &editor in editors {
-        let path = Path::new(editor);
-        if can_execute(path) {
-            return Ok(path.to_owned());
-        }
-    }
-
-    Err(io::Error::new(
-        io::ErrorKind::NotFound,
-        "cannot find text editor",
-    ))
-}
-
 // To detect potential lock-outs if the user called "sudo visudo".
 // Note that SUDO_USER will normally be set by sudo.
 //
@@ -381,7 +385,7 @@ fn sudo_visudo_is_allowed(mut sudoers: Sudoers, host_name: &Hostname) -> Option<
 }
 
 // Make sure that the first valid response is the "safest" choice
-fn ask_response(prompt: &[u8], valid_responses: &[u8]) -> io::Result<u8> {
+pub(crate) fn ask_response(prompt: &[u8], valid_responses: &[u8]) -> io::Result<u8> {
     let stdin = io::stdin();
     let stdout = io::stdout();
     let mut stderr = io::stderr();
